CREATE TABLE IF NOT EXISTS sqlpage_options(
  session text,
  "type" text,
  name text,
  selected boolean
);
DELETE FROM sqlpage_options
WHERE
  session = sqlpage.cookie('session')

INSERT INTO sqlpage_options(session, type, name, selected)
VALUES (
  sqlpage.cookie('session'),
  'start',
  $start,
  TRUE
), (
  sqlpage.cookie('session'),
  'end',
  $end,
  TRUE
)
;

INSERT INTO sqlpage_options
SELECT
  sqlpage.cookie('session'),
  CASE
  WHEN ut.name = 'Volume' THEN 'unit_volume' 
  WHEN ut.name = 'Mass' THEN 'unit_mass' 
  WHEN ut.name = 'Count' THEN 'unit_count' 
  END,
  u.name,
  CASE
  WHEN ut.name = 'Volume' THEN u.name = COALESCE($unit_volume, 'L')
  WHEN ut.name = 'Mass' THEN u.name = COALESCE($unit_mass, 'kg')
  WHEN ut.name = 'Count' THEN CASE WHEN $unit_count IS NOT NULL THEN f.v IS NOT NULL ELSE NULL END
  END
FROM units u
JOIN unit_types ut
ON ut.id = u.unit_type_id
LEFT JOIN json_array_elements($unit_count::json) f(v)
ON u.name = f.v#>>'{}' AND ut.name = 'Count'
;

INSERT INTO sqlpage_options
SELECT
  sqlpage.cookie('session'),
  'tag',
  t.name,
  CASE WHEN $tags IS NOT NULL THEN f.v IS NOT NULL ELSE NULL END
FROM tags t
LEFT JOIN json_array_elements($tags::json) f(v)
ON t.name = f.v#>>'{}'
;

INSERT INTO sqlpage_options
SELECT
  sqlpage.cookie('session'),
  UNNEST(ARRAY['product', 'category', 'group']) AS "type",
  UNNEST(q.name) AS name,
  UNNEST(selected)
FROM (
  SELECT
    ARRAY[p.name, c.name, g.name] AS name,
    ARRAY[
      CASE WHEN $products IS NOT NULL THEN fp.v IS NOT NULL ELSE NULL END,
      CASE WHEN $categories IS NOT NULL THEN fc.v IS NOT NULL ELSE NULL END,
      CASE WHEN $groups IS NOT NULL THEN fg.v IS NOT NULL ELSE NULL END
    ]
  FROM products p
  LEFT JOIN json_array_elements($products::json) fp(v)
  ON p.name = fp.v#>>'{}'
  JOIN categories c ON p.category_id = c.id
  LEFT JOIN json_array_elements($categories::json) fc(v)
  ON c.name = fc.v#>>'{}'
  JOIN groups g ON c.group_id = g.id
  LEFT JOIN json_array_elements($groups::json) fg(v)
  ON g.name = fg.v#>>'{}'
  WHERE (
    (p.name = fp.v#>>'{}' OR $products IS NULL)
  AND
    (c.name = fc.v#>>'{}' OR $categories IS NULL)
  AND
    (g.name = fg.v#>>'{}' OR $groups IS NULL)
  )
) q(name, selected)
UNION
SELECT
  sqlpage.cookie('session'),
  'group',
  f.v#>>'{}',
  TRUE
FROM json_array_elements($groups::json) f(v)
LEFT JOIN groups g ON g.name = f.v#>>'{}'
WHERE g.name is NULL
UNION
SELECT
  sqlpage.cookie('session'),
  'category',
  f.v#>>'{}',
  TRUE
FROM json_array_elements($categories::json) f(v)
LEFT JOIN categories c ON c.name = f.v#>>'{}'
WHERE c.name is NULL
UNION
SELECT
  sqlpage.cookie('session'),
  'product',
  f.v#>>'{}',
  TRUE
FROM json_array_elements($products::json) f(v)
LEFT JOIN products p ON p.name = f.v#>>'{}'
WHERE p.name is NULL
;
